<?php
/**
 * @version		2.0.0
 * @package		WebRotate 360 Product Viewer (plugin)
 * @author      WebRotate 360 LLC
 * @copyright	Copyright (C) WebRotate 360 LLC. All rights reserved.
 * @license		GNU General Public License version 2 or later (http://www.gnu.org/copyleft/gpl.html).
 */

defined("_JEXEC") or die;

use Joomla\CMS\Factory;
use Joomla\Component\Fields\Administrator\Helper\FieldsHelper;

class PlgContentWebRotate360 extends JPlugin
{
    private $viewerWidth;
    private $viewerHeight;
    private $baseWidth;
    private $minHeight;
    private $aspectRatio;
    private $config;
    private $video;
    private $viewerSkin;
    private $popupSkin;
    private $licensePath;
    private $name;
    private $callback;
    private $graphicsPath;
    private $background;
    private $rootPath;
    private $isPopupGallery;
    private $inBrowserFullscreen;
    private $useGoogleEvents;
    private $title;
    private $headerInitialized = false;

    function __construct(&$subject, $config)
    {
        parent::__construct($subject, $config);
    }

    public function onContentPrepare($context, &$article, &$params, $page = 0)
    {
        $parts = explode(".", $context);

        $contentMode = $parts[0];

        if (($contentMode !== "com_content")    &&
            ($contentMode !== "mod_k2_content") &&
            ($contentMode !== "com_k2")         &&
            ($contentMode !== "mod_custom")     &&
			($contentMode !== "text")			&&
            ($contentMode !== "com_djcatalog2") &&
            ($contentMode !== "com_virtuemart")) {
            return false;
        }

        $isShortcodes = str_contains($article->text, '[wr360');
        $isFields = str_contains($article->text, 'field');

        if (empty($article->text) || (!$isShortcodes && !$isFields)) {
            return false;
        }

        if (!$this->headerInitialized) {
            $this->initHeaderScripts();
            $this->headerInitialized = true;
        }

        if (isset($article->text)) {
            if ($isShortcodes) {
                $article->text = $this->processCodes($article->text);
            }
            if ($isFields) {
                $article->text = $this->processFields($article->text, FieldsHelper::getFields('com_content.article', $article, true));
            }
        }
    }

    private function initHeaderScripts()
    {
        $this->initHeaderDefaults();
        $doc = Factory::getApplication()->getDocument();

        JHtml::_("jquery.framework");

        $doc->addScript(JURI::base() . "plugins/content/webrotate360/imagerotator/html/js/imagerotator.js");
        $doc->addScript(JURI::base() . "plugins/content/webrotate360/prettyphoto/js/jquery.prettyPhoto.js");
        $doc->addScript(JURI::base() . "plugins/content/webrotate360/webrotate360.js");
/*
        $include_i18n_path = $this->getOption("wr360_i18n_path");
        if (!empty($include_i18n_path)) {
            $doc->addScript($include_i18n_path);
        }
*/
        $doc->addStyleSheet(JURI::base() . "plugins/content/webrotate360/prettyphoto/css/prettyphoto.css");
        $doc->addStyleSheet(JURI::base() . "plugins/content/webrotate360/imagerotator/html/css/" . $this->viewerSkin . ".css");
        $doc->addStyleSheet(JURI::base() . "plugins/content/webrotate360/webrotate360.css");

        $helperScript = "function getWR360PopupSkin(){return '" . $this->popupSkin . "';}";

        $doc->addScriptDeclaration($helperScript);
    }

    private function processFields($content, $customFields)
    {
        $regex = '/{(field)\s+(.*?)}/i';

        preg_match_all($regex, $content, $matches, PREG_SET_ORDER);
        if (!$matches)  {
            return $content;
        }

        $fieldsById = [];
        foreach ($customFields as $field) {
            $fieldsById[$field->id] = $field;
        }

        foreach ($matches as $i => $match) {
            $explode = explode(',', $match[2]);
            $id = (int)$explode[0];

            if ($match[1] === 'field' && $id) {
                $field = $fieldsById[$id];
                if (isset($field)) {
                    if ($field->name === "webrotate360-config") {
                        $content = preg_replace("|$match[0]|", $this->embedShortcodeHanlder(["name" => $field->name, "config" => $field->value]), $content, 1);
                    } else if ($field->name === "webrotate360-video") {
                        $content = preg_replace("|$match[0]|", $this->videoShortcodeHanlder(["name" => $field->name, "video" => $field->value]), $content, 1);
                    }
                }
            }
        }

        return $content;
    }

    private function processCodes($content)
    {
        $regex = '/(.?)\[(wr360embed|wr360popup|wr360expand|wr360video)\b(.*?)(?:(\/))?\](?:(.+?)\[\/\2\])?(.?)/s';

        return (preg_replace_callback($regex, array($this, "replaceCodes"), $content));
    }

    private function replaceCodes($codeMatch)
    {
        $codeId      = $codeMatch[2];
        $codeStr     = $codeMatch[3];
        $codeBefore  = $codeMatch[1];
        $codeAfter   = $codeMatch[6];
        $codeContent = $codeMatch[5];

        $attribs = [];
        if (preg_match_all('/([^,= ]+)=("[^"]+")/', $codeStr, $attribMatch)) {
            $attribs = array_combine($attribMatch[1], $attribMatch[2]);
        }

        if (!empty($codeContent)) {
            if ($codeId === "wr360popup")
                $replacer = $this->popupShortcodeHanlder($attribs, $codeContent);
            else if ($codeId === "wr360expand")
                $replacer = $this->expandShortcodeHanlder($attribs, $codeContent);
        } else {
            if ($codeId === "wr360embed")
                $replacer = $this->embedShortcodeHanlder($attribs);
            else if ($codeId === "wr360video")
                $replacer = $this->videoShortcodeHanlder($attribs);
        }

        if (!empty($replacer))
            return ($codeBefore . $replacer . $codeAfter);

        return "";
    }

    private function videoShortcodeHanlder($attribs)
    {
        $this->initShortcode($attribs);
        $videoWidth = $this->viewerWidth;
        $videoHeight = $this->aspectRatio === "auto" ? $this->viewerHeight : "auto";

        $replace  = "<div id='%s' style='width:%s; aspect-ratio:%s; height:%s' class='webrotate360 wr360video'>";
        $replace .= '<iframe allowfullscreen src="%s">';
        $replace .= "</iframe>";
        $replace .= "</div>";

        return sprintf(
            $replace,
            "wr360_" . $this->name,
            !str_contains($videoWidth, "%") ? $videoWidth . "px" : $videoWidth,
            $this->aspectRatio,
            !str_contains($videoHeight, "%") && $videoHeight !== "auto" ? $videoHeight . "px" : $videoHeight,
            $this->video);
    }

    private function expandShortcodeHanlder($attribs, $content)
    {
        $this->initShortcode($attribs);

        $replace  = "<div id='%s' style='cursor:pointer;' class='webrotate360 wr360embed' data-imagerotator='{";
        $replace .= '"onready":"%s", "graphics":"%s", "licfile":"%s", "rootpath":"%s", "xmlfile":"%s", "events":%s, "name":"%s", "fsclick":true, "browserfs":%s';
        $replace .= "}'>";
        $replace .= $content;
        $replace .= "</div>";

        return sprintf(
            $replace,
            "wr360_" . $this->name,
            $this->callback,
            $this->graphicsPath,
            $this->licensePath,
            $this->rootPath,
            $this->config,
            $this->useGoogleEvents ? "true" : "false",
            $this->name,
            $this->inBrowserFullscreen === "true" ? "true" : "false");
    }

    private function popupShortcodeHanlder($attribs, $content)
    {
        $this->initShortcode($attribs);

        $replace  = "<a id='%s' title='%s' href='%s?config=%s&amp;root=%s&amp;height=%s&amp;viewname=%s&amp;grphpath=%s&amp;analyt=%s&amp;lic=%s&amp;background=%s&amp;iframe=true&amp;width=%s&amp;height=%s' data-rel=prettyPhoto%s>";
        $replace .= $content;
        $replace .= "</a>";

        return sprintf(
            $replace,
            "wr360_" . $this->name,
            $this->title,
            JURI::base() . "plugins/content/webrotate360/viewloader_" . $this->viewerSkin . ".html",
            urlencode($this->config),
            urlencode($this->rootPath),
            $this->viewerHeight,
            $this->name,
            urlencode($this->graphicsPath),
            $this->useGoogleEvents ? "true" : "false",
            urlencode($this->licensePath),
            urlencode($this->background),
            str_ireplace("%", "", $this->viewerWidth),
            $this->viewerHeight,
            $this->isPopupGallery ? "[iframe]" : ""
        );
    }

    private function embedShortcodeHanlder($attribs)
    {
        $this->initShortcode($attribs);

        $usingAspect = $this->aspectRatio !== "auto";

        $viewerWidth = $this->viewerWidth;
        $viewerHeight = $usingAspect ? "100%" : $this->viewerHeight;
        $baseWidth = $usingAspect ? 0 : $this->baseWidth;
        $minHeight = $usingAspect ? 0 : $this->minHeight;

        $replace  = "<div id='%s' class='webrotate360 wr360embed' style='aspect-ratio:%s; width:%s; height:%s;' data-imagerotator='{";
        $replace .= '"onready":"%s", "graphics":"%s", "licfile":"%s", "rootpath":"%s", "xmlfile":"%s", "basewidth":%s, "events":%s, "name":"%s", "minheight":%s, "background":"%s"';
        $replace .= "}'></div>";

        return sprintf(
            $replace,
            'wr360_' . $this->name,
            $this->aspectRatio,
            !str_contains($viewerWidth, '%') ? $this->viewerWidth . 'px' : $this->viewerWidth,
            !str_contains($viewerHeight, "%") ? $this->viewerHeight . "px" : $this->viewerHeight,
            $this->callback,
            $this->graphicsPath,
            $this->licensePath,
            $this->rootPath,
            $this->config,
            $baseWidth,
            $this->useGoogleEvents ? "true" : "false",
            $this->name,
            $minHeight,
            $this->background);
    }

    private function initShortcode($attribs)
    {
        $extract = $this->initShortcodeDefaults([
            "name"         => "",
            "width"        => "",
            "height"       => "",
            "rootpath"     => "",
            "config"       => "",
            "video"        => "",
            "aspectRatio"  => "",
            "basewidth"    => "",
            "minheight"    => "",
            "gallery"      => "false",
            "viewerskin"   => "",
            "background"   => "",
            "browserfs"    => "",
            "callback"     => "",
            "title"        => ""], $attribs);

        $this->viewerWidth  = str_ireplace("px", "", $this->getOption("wr360_viewer_width"));
        $this->viewerHeight = str_ireplace("px", "", $this->getOption("wr360_viewer_height"));
        $this->baseWidth    = str_ireplace("px", "", $this->getOption("wr360_viewer_basewidth"));
        $this->minHeight    = str_ireplace("px", "", $this->getOption("wr360_viewer_minheight"));
        $this->callback     = $this->getOption("wr360_api_ready_callback");
        $this->config       = $this->getOption("wr360_master_config");
        $this->background   = $this->getOption("wr360_background_color");
        $this->graphicsPath = $this->getOption("wr360_graphics_path");
        $this->aspectRatio  = $this->getOption("wr360_size_aspect");

        if (empty($this->viewerWidth))
            $this->viewerWidth = 300;
        if (empty($this->viewerHeight))
            $this->viewerHeight = 300;
        if (empty($this->baseWidth))
            $this->baseWidth = 0;
        if (empty($this->minHeight))
            $this->minHeight = 0;
        if (empty($this->graphicsPath))
            $this->graphicsPath = JURI::base() . "plugins/content/webrotate360/graphics";
        if (empty($this->aspectRatio))
            $this->aspectRatio = "auto";

        if (!empty($extract["width"]))
            $this->viewerWidth = str_ireplace("px", "", $extract["width"]);
        if (!empty($extract["height"]))
            $this->viewerHeight = str_ireplace("px", "", $extract["height"]);
        if (!empty($extract["basewidth"]))
            $this->baseWidth = str_ireplace("px", "", $extract["basewidth"]);
        if (!empty($extract["minheight"]))
            $this->minHeight = str_ireplace("px", "", $extract["minheight"]);
        if (!empty($extract["config"]))
            $this->config = $extract["config"];
        if (!empty($extract["viewerskin"]))
            $this->viewerSkin = $extract["viewerskin"];
        if (!empty($extract["callback"]))
            $this->callback = $extract["callback"];
        if (!empty($extract["background"]))
            $this->background = $extract["background"];
        if (!empty($extract["browserfs"]))
            $this->inBrowserFullscreen = $extract["browserfs"];
        if (!empty($extract["title"]))
            $this->title = $extract["title"];
        if (!empty($extract["video"]))
            $this->video = $extract["video"];
        if (!empty($extract["aspectRatio"]))
            $this->aspectRatio = $extract["aspectRatio"];

        // Just to keep things short in script.
        if (empty($this->callback))
            $this->callback = "wr360_callback_not_set";

        $this->name = preg_replace('/\s+/', '', $extract["name"]);
        $this->rootPath = $extract["rootpath"];

        $this->isPopupGallery = $extract["gallery"];
        if (!empty($this->isPopupGallery))
            $this->isPopupGallery = ($this->isPopupGallery === "true");
    }

    private function initHeaderDefaults()
    {
        $this->viewerSkin = $this->getOption("wr360_viewer_skin");
        if (empty($this->viewerSkin))
            $this->viewerSkin = "basic";

        $this->popupSkin = $this->getOption("wr360_popup_skin");
        if (empty($this->popupSkin) || ($this->popupSkin == "default"))
            $this->popupSkin = "pp_default";

        $this->licensePath = $this->getOption("wr360_license_path");
        if (empty($this->licensePath))
            $this->licensePath = JURI::base() . "plugins/content/webrotate360/license.lic";

        $this->useGoogleEvents = $this->getOption("wr360_use_googleevents");
        if (!empty($this->useGoogleEvents))
            $this->useGoogleEvents = true;
    }

    private function initShortcodeDefaults($defaults, $attribs)
    {
        $parsed = [];
        foreach($defaults as $name => $value) {
            if (array_key_exists($name, $attribs)) {
                $parsed[$name] = trim($attribs[$name], '"');
                continue;
            }

            $parsed[$name] = $value;
        }

        return ($parsed);
    }

    private function getOption($optionName)
    {
        return ($this->params->get($optionName));
    }
}
